#pragma once

#include <amdis/Output.hpp>

namespace AMDiS
{
  /// Interface for Preconditioner types
  template <class Traits>
  struct PreconditionerInterface
  {
    using Mat = typename Traits::Mat;
    using Sol = typename Traits::Sol;
    using Rhs = typename Traits::Rhs;

    /// Virtual destructor.
    virtual ~PreconditionerInterface() = default;

    /// Is called a the beginning of a solution procedure
    virtual void init(Mat const& fullMatrix) = 0;

    /// Is called at the end of a solution procedure
    virtual void exit() = 0;

    /// Apply the preconditioner to a vector \p b and store the result in \p x
    virtual void solve(Rhs const& b, Sol& x) const = 0;

    /// Apply the transposed preconditioner to a vector \p b and store the result in \p x
    virtual void adjoint_solve(Rhs const& b, Sol& x) const
    {
      error_exit("Must be implemented by derived class.");
    }
  };

} // end namespace AMDiS
