#include <amdis/AMDiS.hpp>
#include <amdis/AdaptStationary.hpp>
#include <amdis/LocalOperators.hpp>
#include <amdis/ProblemStat.hpp>

using namespace AMDiS;

using Grid = Dune::YaspGrid<2>;

template <class T>
struct Param
    : DefaultBasisCreator<Grid, Impl::LagrangePreBasisCreator<1,1>, T>
{};

template <class T>
void test()
{
  ProblemStat<Param<T>> prob("ellipt");
  prob.initialize(INIT_ALL);
  prob.boundaryManager()->setBoxBoundary({1,1,2,2});

  prob.addMatrixOperator(sot(T(1)), 0, 0);
  prob.addVectorOperator(zot(T(1)), 0);
  prob.addDirichletBC(BoundaryType{1}, 0,0, T(0));


  AdaptInfo adaptInfo("adapt");

  // test copy constructor of problem
  auto prob2(prob);
  AdaptStationary adaptStat2("adapt", prob2, adaptInfo);
  adaptStat2.adapt();

  // test move constructor of problem
  auto prob3(std::move(prob2));
  AdaptStationary adaptStat3("adapt", prob3, adaptInfo);
  adaptStat3.adapt();
}

int main(int argc, char** argv)
{
  Environment env(argc, argv);

  test<float>();
  test<double>();
  test<long double>();

  return 0;
}
